﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1 {
    class Program {
        static void Main(string[] args) {

            //This is a simple C# Console project to illustrate that VisualAPL scripts can be used
            //directly from any .Net language. VisualAPL scripts can contain data as VisualAPL variables and
            //methods as VisualAPL fuctions.

            //VisualAPL scripts are an available alternative to compiling VisualAPL source code into a
            //.Net assembly.

            //A VisualAPL script is non-proprietary Unicode text file, editable with Microsoft Notepad.
            //From the Microsoft Visual Studio Solution Explorer in this C# project, double click the
            //VisualAPL "SampleScript.txt" file to view its contents in the Visual Studio text editor.

            //For proper operation of this C# project, the references may need updating to indicate the
            //location of the VisualAPL assemblies used. When VisualAPL is installed on the programmer's
            //machine, these assemblies are installed to the programmer-selected location.

            Console.WriteLine("Using VisualAPL Scripts from C#:");
            
            APLNext.CieloInteractive.Hosting.APLEngine aplEngine = new APLNext.CieloInteractive.Hosting.APLEngine();
            //^Create an instance of the APLEngine which powers the Cielo Explorer in Visual Studio,
            // but can also be used independently to use VisualAPL from any .Net language

            object res = aplEngine.Evaluate("1+1");
            //^Use any APL operator from this C# project and return the result to this C# project

            aplEngine.Execute("a = 1 2 3 4");
            //^Create a VisualAPL array (a simple one in this case) from this C# project

            object a = aplEngine.GetVariable("a");
            //^Obtain a VisualAPL object for possible further processing in this C# project

            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            //^Obtain this assembly
            string resourceName = "ConsoleApplication1.VisualAPLScripts.SampleScript.txt";
            //^The VisualAPL SampleScript.txt file has been added to this C# project
            // with BuildAction property set to EmbeddedResource
            System.IO.TextReader textReader = new System.IO.StreamReader(assembly.GetManifestResourceStream(resourceName));
            string textcode = textReader.ReadToEnd();
            textReader.Close();
            //^Obtain the contents of the SampleScript.txt file

            object objectcode = aplEngine.Compile(textcode);
            //^Compile the VisualAPL script
            aplEngine.Execute(objectcode);
            //^Execute the compiled code, which defines the VisualAPL functions 'add' and 'minus' from the script
            
            res = aplEngine.Evaluate("add(10,20)");
            //^Use the VisualAPL function 'add' on scalar arguments and return the value to this C# project
            Console.WriteLine("Result of VisualAPL function: add(10,20): "+res.ToString());

            aplEngine.SetVariable("d", new int[] { 10, 20, 30 });
            aplEngine.SetVariable("e", new int[] { 5, 10, 15 });
            //^Define two VisualAPL arrays
            res = aplEngine.Evaluate("minus(d,e)");
            //^Use the VisualAPL function 'minus' on array arguments and return the result to this C# project
            Console.WriteLine("Result of VisualAPL function: minus(10 20 30, 5 10 15): "+res.ToString());

            Console.Read();
            //^Wait for user to close console window
        }
    }
}